@@ -48,7 +48,7 @@
     </message>
     <message>
         <source>Toggle</source>
-        <translation>Seçimi ter çevir</translation>
+        <translation>Seçimi ters çevir</translation>
     </message>
 </context>
 <context>
@@ -527,6 +527,14 @@
         <source> (remember to close it again to avoid access problems)</source>
         <translation> (erişim sorunlarını önlemek için tekrar kapatmayı unutmayın)</translation>
     </message>
+    <message>
+        <source>Settings and Resources...</source>
+        <translation>Ayarlar ve Kaynaklar...</translation>
+    </message>
+    <message>
+        <source>Clear Recent Files</source>
+        <translation>Son kullanılanları temizle</translation>
+    </message>
 </context>
 <context>
     <name>PDFFindDialog</name>
@@ -839,7 +847,7 @@
     </message>
     <message>
         <source>Auto-hide output panel unless errors occur</source>
-        <translation>Hata bulunmazsa çıktı panelini gizle</translation>
+        <translation type="obsolete">Hata bulunmazsa çıktı panelini gizle</translation>
     </message>
     <message>
         <source>Line numbers</source>
@@ -875,7 +883,7 @@
     </message>
     <message>
         <source>Allow scripts to write files</source>
-        <translation>Eklentileri dosya yazmasına izin ver</translation>
+        <translation>Eklentilerin dosya yazmasına izin ver</translation>
     </message>
     <message>
         <source>Allow scripts to run system commands</source>
@@ -893,6 +901,57 @@
         <source>Enable QtScript debugger</source>
         <translation>QtScript hata ayıklayıcısını etkinleştir</translation>
     </message>
+    <message>
+        <source>Hide output panel:</source>
+        <translation type="obsolete">Çıktı panelini gizle:</translation>
+    </message>
+    <message>
+        <source>Never</source>
+        <extracomment>&quot;Hide console output&quot; mode</extracomment>
+        <translation>Hiçbir zaman</translation>
+    </message>
+    <message>
+        <source>Automatically</source>
+        <extracomment>&quot;Hide console output&quot; mode</extracomment>
+        <translation>Otomatik olarak</translation>
+    </message>
+    <message>
+        <source>On success</source>
+        <extracomment>&quot;Hide console output&quot; mode</extracomment>
+        <translation>Hata yoksa</translation>
+    </message>
+    <message>
+        <source>&lt;em&gt;Note:&lt;/em&gt; These are default settings. Changes to them will not affect windows that are already open. Use the respective items in the &amp;quot;Format&amp;quot; menu to achieve that.</source>
+        <translation>&lt;em&gt;Not:&lt;/em&gt; Bunlar öntanımlı ayarlardır. Yapılacak değişiklikler halen açık olan pencereleri etkilemez. O amaçla &amp;quot;Biçim&amp;quot; menüsündeki ilgili seçenekleri kullanın.</translation>
+    </message>
+    <message>
+        <source>&lt;p&gt;&lt;strong&gt;Never:&lt;/strong&gt; Never hide the output panel after typesetting.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;Automatically:&lt;/strong&gt; Hide the output panel after successful typesetting, but only if it was hidden before.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;On Success:&lt;/strong&gt; Hide the output panel after successful typesetting, regardless of its previous state.&lt;/p&gt;
+</source>
+        <translation type="obsolete">&lt;p&gt;&lt;strong&gt;Hiçbir zaman:&lt;/strong&gt; Dizgi sonrası çıktı panelini hiçbir zaman gizleme.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;Otomatik olarak:&lt;/strong&gt; Dizgi başarılı ise çıktı panelini eğer önceden zaten gizli ise gizle.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;Hata yoksa:&lt;/strong&gt; Dizgi başarılı ise çıktı panelini önceki durumunu dikkate almaksızın gizle.&lt;/p&gt;
+</translation>
+    </message>
+    <message>
+        <source>Enable autocompletion</source>
+        <translation>Kelime tamamlamayı etkinleştir</translation>
+    </message>
+    <message>
+        <source>Hide console output:</source>
+        <translation>Konsol çıktısını gizle:</translation>
+    </message>
+    <message>
+        <source>&lt;p&gt;&lt;strong&gt;Never:&lt;/strong&gt; Never hide the console output panel after typesetting.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;Automatically:&lt;/strong&gt; Hide the console output panel after successful typesetting, but only if it was hidden before.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;On Success:&lt;/strong&gt; Hide the console output panel after successful typesetting, regardless of its previous state.&lt;/p&gt;
+</source>
+        <translation>&lt;p&gt;&lt;strong&gt;Asla:&lt;/strong&gt; Dizgi sonrası konsol çıktısını asla gizleme.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;Otomatik:&lt;/strong&gt; Hatasız dizgi sonrası konsol çıktısını şayet önceden de gizli ise gizle.&lt;/p&gt;
+&lt;p&gt;&lt;strong&gt;Hata yoksa:&lt;/strong&gt; Hatasız dizgi sonrası konsol çıktısını önceki durumunu dikkate almadan gizle.&lt;/p&gt;
+</translation>
+    </message>
 </context>
 <context>
     <name>QObject</name>
@@ -922,7 +981,7 @@
     </message>
     <message>
         <source>Auxiliary files (*.aux *.toc *.lot *.lof *.nav *.out *.snm *.ind *.idx *.bbl *.log)</source>
-        <translation>Yardımcı dosyalar (*.aux *.toc *.lot *.lof *.nav *.out *.snm *.ind *.idx *.bbl *.log)</translation>
+        <translation type="obsolete">Yardımcı dosyalar (*.aux *.toc *.lot *.lof *.nav *.out *.snm *.ind *.idx *.bbl *.log)</translation>
     </message>
     <message>
         <source>Text files (*.txt)</source>
@@ -942,6 +1001,14 @@
         <translation type="obsolete">TeXworks %1r%2 (%3)
 </translation>
     </message>
+    <message>
+        <source>Log files (*.log)</source>
+        <translation>Kayıt dosyaları (*.log)</translation>
+    </message>
+    <message>
+        <source>Auxiliary files (*.aux *.toc *.lot *.lof *.nav *.out *.snm *.ind *.idx *.bbl)</source>
+        <translation>Yardımcı dosyalar (*.aux *.toc *.lot *.lof *.nav *.out *.snm *.ind *.idx *.bbl)</translation>
+    </message>
 </context>
 <context>
     <name>ReplaceDialog</name>
@@ -1006,6 +1073,29 @@
         <translation>&amp;Düzenli ifade</translation>
     </message>
 </context>
+<context>
+    <name>ResourcesDialog</name>
+    <message>
+        <source>TeXworks Settings and Resources</source>
+        <translation>TeXworks Ayarları ve Kaynaklar</translation>
+    </message>
+    <message>
+        <source>Storage Locations</source>
+        <translation>Depolama Yerleri</translation>
+    </message>
+    <message>
+        <source>Settings:</source>
+        <translation>Ayarlar:</translation>
+    </message>
+    <message>
+        <source>Resources:</source>
+        <translation>Kaynaklar:</translation>
+    </message>
+    <message>
+        <source>Registry (%1)</source>
+        <translation>Kütük (%1)</translation>
+    </message>
+</context>
 <context>
     <name>ScriptManager</name>
     <message>
@@ -1156,7 +1246,7 @@
     </message>
     <message>
         <source>&lt;p&gt;Distributed under the &lt;a href=&quot;http://www.gnu.org/licenses/gpl-2.0.html&quot;&gt;GNU General Public License&lt;/a&gt;, version 2.</source>
-        <translation>&lt;p&gt;&lt;a href=&quot;http://www.gnu.org/licenses/gpl-2.0.html&quot;&gt;GNU Genel Kamu Lisansı&lt;/a&gt;nın 2. versiyonu ile dağıtılır.</translation>
+        <translation type="obsolete">&lt;p&gt;&lt;a href=&quot;http://www.gnu.org/licenses/gpl-2.0.html&quot;&gt;GNU Genel Kamu Lisansı&lt;/a&gt;nın 2. versiyonu ile dağıtılır.</translation>
     </message>
     <message>
         <source>&lt;p&gt;&lt;a href=&quot;http://trolltech.com/products/&quot;&gt;Qt4&lt;/a&gt; application framework by Qt Software, a division of Nokia Corporation.</source>
@@ -1246,6 +1336,14 @@
         <source>The script &quot;%1&quot; is trying to open the file &quot;%2&quot; without sufficient permissions. Do you want to open the file?</source>
         <translation>&quot;%1&quot; eklentisi &quot;%2&quot; dosyasını yeterli izin olmadan açmaya çalışıyor. Dosyayı açmak istiyor musunuz?</translation>
     </message>
+    <message>
+        <source>&lt;p&gt;Distributed under the &lt;a href=&quot;http://www.gnu.org/licenses/gpl-2.0.html&quot;&gt;GNU General Public License&lt;/a&gt;, version 2 or (at your option) any later version.</source>
+        <translation>&lt;p&gt;&lt;a href=&quot;http://www.gnu.org/licenses/gpl-2.0.html&quot;&gt;GNU Genel Kamu Lisansı&lt;/a&gt;&apos;nın 2. sürümü veya (isteğinize göre) daha sonraki bir sürümü altında dağıtılır.</translation>
+    </message>
+    <message>
+        <source>Clear Recent Files</source>
+        <translation>Son kullanılanları temizle</translation>
+    </message>
 </context>
 <context>
     <name>TWScriptAPI</name>
@@ -1541,11 +1639,11 @@ Tercihler penceresinden %2 aracının ayarlarını ve arama yolu yapılandırmas
     </message>
     <message>
         <source>Hide Output Panel</source>
-        <translation>Çıktı Panelini Gizle</translation>
+        <translation type="obsolete">Çıktı Panelini Gizle</translation>
     </message>
     <message>
         <source>Show Output Panel</source>
-        <translation>Çıktı Panelini Göster</translation>
+        <translation type="obsolete">Çıktı Panelini Göster</translation>
     </message>
     <message>
         <source>Set engine to &quot;%1&quot;</source>
@@ -1941,7 +2039,7 @@ Tercihler penceresinden %2 aracının ayarlarını ve arama yolu yapılandırmas
     </message>
     <message>
         <source>Check configuration of the %1 tool and path settings in the Preferences dialog.</source>
-        <translation>Tercihler penceresinden %1 aracının ayarlarını ve arama yolu yapılandırmasını kontrol edin.</translation>
+        <translation type="obsolete">Tercihler penceresinden %1 aracının ayarlarını ve arama yolu yapılandırmasını kontrol edin.</translation>
     </message>
     <message>
         <source>Line Numbers</source>
@@ -2055,6 +2153,96 @@ Haricî süreç tamamlandığında lütfen &quot;Dosya -&gt; Kaydedilene Geri D
         <source>Abort typesetting</source>
         <translation>Dizgiyi durdur</translation>
     </message>
+    <message>
+        <source>Settings and Resources...</source>
+        <translation>Ayarlar ve Kaynaklar...</translation>
+    </message>
+    <message>
+        <source>Reload using selected encoding</source>
+        <extracomment>Item in the encoding popup menu</extracomment>
+        <translation>Seçili kodlama ile tekrar yükle</translation>
+    </message>
+    <message>
+        <source>Reloads the current file with the encoding selected from this menu.
+
+The selected encoding replaces the default one and overrides all &quot;%!TEX encoding&quot; lines.</source>
+        <extracomment>Tooltip for &quot;Reload using selected encoding&quot;</extracomment>
+        <translation>Mevcut dosyayı bu menüden seçilecek kodlama ile tekrar yükler.
+
+Seçili kodlama öntanımlının yerine geçer ve tüm &quot;%!TEX encoding&quot; satırlarını geçersiz kılar.</translation>
+    </message>
+    <message>
+        <source>Unsaved changes</source>
+        <translation>Kaydedilmemiş değişiklikler</translation>
+    </message>
+    <message>
+        <source>The file you are trying to reload has unsaved changes.
+
+Do you want to discard your current changes, and reload the file from disk with the encoding %1?</source>
+        <translation>Tekrar yüklemeye çalıştığınız dosyada kaydedilmemiş değişiklikler var.
+
+Mevcut değişikliklerinizi gözardı etmek, ve dosyayı diskten %1 kodlaması ile tekrar yüklemek istiyor musunuz?</translation>
+    </message>
+    <message>
+        <source>Clear Recent Files</source>
+        <translation>Son kullanılanları temizle</translation>
+    </message>
+    <message>
+        <source>Abort typesetting?</source>
+        <translation>Dizgi iptal edilsin mi?</translation>
+    </message>
+    <message>
+        <source>A typesetting process is still running and must be stopped before closing this window.
+Do you want to stop it now?</source>
+	<translation>Bir dizgi işlemi halen devam ediyor ve bu pencereyi kapatmadan önce durdurulması gerekli.
+Şimdi durdurmak istiyor musunuz?</translation>
+    </message>
+    <message>
+        <source>You need a &lt;b&gt;TeX distribution&lt;/b&gt; like &lt;a href=&quot;http://tug.org/texlive/&quot;&gt;TeX Live&lt;/a&gt; or &lt;a href=&quot;http://miktex.org/&quot;&gt;MiKTeX&lt;/a&gt; installed on your system to typeset your document.</source>
+        <translation>Belgenizi dizmek için sisteminize &lt;a href=&quot;http://tug.org/texlive/&quot;&gt;TeX Live&lt;/a&gt; veya &lt;a href=&quot;http://miktex.org/&quot;&gt;MiKTeX&lt;/a&gt; gibi bir &lt;b&gt;TeX dağıtımı&lt;/b&gt; kurmanız gerekiyor.</translation>
+    </message>
+    <message>
+        <source>You need a &lt;b&gt;TeX distribution&lt;/b&gt; like &lt;a href=&quot;http://www.tug.org/mactex/&quot;&gt;MacTeX&lt;/a&gt; installed on your system to typeset your document.</source>
+        <translation>Belgenizi dizmek için sisteminize &lt;a href=&quot;http://www.tug.org/mactex/&quot;&gt;MacTeX&lt;/a&gt; gibi bir &lt;b&gt;TeX dağıtımı&lt;/b&gt; kurmanız gerekiyor.</translation>
+    </message>
+    <message>
+        <source>You need a &lt;b&gt;TeX distribution&lt;/b&gt; like &lt;a href=&quot;http://tug.org/texlive/&quot;&gt;TeX Live&lt;/a&gt; installed on your system to typeset your document. On most systems such a TeX distribution is available as prebuilt package.</source>
+        <translation>Belgenizi dizmek için sisteminize &lt;a href=&quot;http://tug.org/texlive/&quot;&gt;TeX Live&lt;/a&gt; gibi bir &lt;b&gt;TeX dağıtımı&lt;/b&gt; kurmanız gerekiyor. Çoğu sistem için böyle bir TeX dağıtımı hazır paket şeklinde mevcuttur.</translation>
+    </message>
+    <message>
+        <source>When a TeX distribution is installed you may need to tell TeXworks where to find it in Edit -&gt; Preferences -&gt; Typesetting.</source>
+        <translation>Bir TeX dağıtımı kurulduğunda TeXworks'e onu nerede bulabileceğini Düzenle -&gt; Tercihler -&gt; Dizgi ile anlatmanız gerekebilir.</translation>
+    </message>
+    <message>
+        <source>Check the configuration of the %1 tool and the path settings in the Preferences dialog.</source>
+        <translation>%1 aracının konfigürasonunu ve yol ayarlarını Tercihler penceresinden kontrol edit.</translation>
+    </message>
+    <message>
+        <source>Hide Console Output</source>
+        <translation>Konsol Çıktısını Gizle</translation>
+    </message>
+    <message>
+        <source>Layout Problem</source>
+        <translation>Yerleşim Problemi</translation>
+    </message>
+    <message>
+        <source>A problem occured while laying out the loaded document in the editor. This is caused by an issue in the underlying Qt framework and can cause TeXworks to crash under certain circumstances. The symptoms of this problem are hidden or overlapping lines. To work around this, please try one of the following:
+ -) Turn syntax highlighting off and on
+ -) Turn line numbers off and on
+ -) Resize the window
+
+We are sorry for the inconvenience.</source>
+	<translation>Yüklenen belgeyi düzenleyiciye yerleştirirken bir sorun ortaya çıktı. Bu Qt çatısından kaynaklanıyor ve bazı durumlarda TeXworks'ün çökmesine neden olabilir. Bu sorun gizli veya üst üste binen satırlar olarak gözlenebilir. Aşağıdakilerden birini deneyerek bunun üstesinden gelebilirsiniz:
+ -) Sentaks renklendirmeyi kapatıp açın
+ -) Satır numaralarını kapatıp açın
+ -) Pencereyi boyutunu değiştirin
+
+Rahatsızlık için üzgünüz.</translation>
+    </message>
+    <message>
+        <source>Show Console Output</source>
+	<translation>Konsol Çıktısını Göster</translation>
+    </message>
 </context>
 <context>
     <name>TeXHighlighter</name>
